/* ****************************************************************************
 *
 * rayme - ray tracing application for virtual 3D scenes 
 *
 * ----------------------------------------------------------------------------
 *
 * Andrej Rypak, xrypak@gmail.com, 2011 - 2012
 *
 * Thsesis project - Raytracing of Virtual Graphics Scenes
 * Brno University of Technology, Faculty of Information Technologies
 * Andrej Rypak, xrypak00@stud.fit.vutbr.cz, 2011/2012,
 * under supervision of Jan Peciva, PhD.
 *
 ******************************************************************************/


#include "EventHandler.h"

using namespace Rayme;


EventHandler::EventHandler(UILayer& ui, RayManager *rm, WindowManager *wm) :
_rm(rm), _wm(wm), _ui(ui) {
	_written = false;
	_select = 0;
}


bool EventHandler::handle(const osgGA::GUIEventAdapter& ea, osgGA::GUIActionAdapter& aa) {
	if (!ea.getHandled() && ea.getEventType() == osgGA::GUIEventAdapter::KEYDOWN) {
		int key = ea.getKey();

		uint t = time(0);
		switch (key) {
			case 'r':
			case osgGA::GUIEventAdapter::KEY_Return:
				_wm->prepareBuffers();
				// manazer pre okna: otvor okno pre raytracing
				if (_rm->isActive()) {
					_wm->closeRayWindow();
				} else {
					_wm->openRayWindow();
				}
				_written = false;

				// manazer pre ray tracing: vytvor snimku
				_rm->requestNewFrame();
				return true;
				break;
			case 'p':
				if (!_rm->isPointTracing()) {
					// manazer pre okna: otvor okno pre raytracing
					_wm->openRayWindow();

					// manazer pre ray tracing: vytvor snimku
					_rm->startPointTracing();
				} else {
					_rm->stopPointTracing();
				}
				return true;
				break;
			case 'c':
				_rm->writeOutput(_wm->getOutputPath(), t);
				_rm->writeSnapshot(_wm->getOutputPath(), t);
				_written = true;
				// no break
			case 'm':
				_rm->storeMasterCamera(_wm->getOutputPath(), t);
				return true;
				break;
			case 'v':
				if (_rm->isActive()) {
					break;
				}
				_ui.enableRecursion(true);
				_ui.enableReflections(!_ui.getUIState().reflections);
				if (!_ui.getUIState().reflections && !_ui.getUIState().refractions) {
					_ui.enableRecursion(false);
				}
				break;
			case 'b':
				if (_rm->isActive()) {
					break;
				}
				_ui.enableRecursion(true);
				_ui.enableRefractions(!_ui.getUIState().refractions);
				if (!_ui.getUIState().reflections && !_ui.getUIState().refractions) {
					_ui.enableRecursion(false);
				}
				break;
				//			case '1':
				//				if (_rm->isActive()) {
				//					break;
				//				}
				//				_ui.enableRecursion(true);
				//				_ui.enableReflections(true);
				//				_ui.enableRefractions(true);
				//				break;
				//			case '2':
				//				if (_rm->isActive()) {
				//					break;
				//				}
				//
				//				_ui.enableRecursion(true);
				//				_ui.enableReflections(false);
				//				_ui.enableRefractions(true);
				//				break;
				//			case '3':
				//				if (_rm->isActive()) {
				//					break;
				//				}
				//				_ui.enableRecursion(true);
				//				_ui.enableReflections(true);
				//				_ui.enableRefractions(false);
				//				break;
				//			case '4':
				//				if (_rm->isActive()) {
				//					break;
				//				}
				//				_ui.enableRecursion(false);
				//				break;
			case 't':
				if (_rm->isActive()) {
					break;
				}
				_ui.enableTextures(!_ui.getUIState().textures);
				break;
			case 'g':
				if (_rm->isActive()) {
					break;
				}
				_ui.enablePhongShading(!_ui.getUIState().phong);
				break;
			case 'a':
				if (_rm->isActive()) {
					break;
				}
				_ui.setMultisampling(_ui.getUIState().samplesPerPixel == 1 ? 4 : _ui.getUIState().samplesPerPixel == 4 ? 8 : 1);
				break;
			case 'q':
				_select = 0;
				std::cout << "Trimmer/select: both" << std::endl;
				break;
			case 'e':
				_select = 1;
				std::cout << "Trimmer/select: reflections" << std::endl;
				break;
			case 'd':
				_select = 2;
				std::cout << "Trimmer/select: refractions" << std::endl;
				break;
			case '0':
			case '1':
			case '2':
			case '3':
			case '4':
			case '5':
			case '6':
			case '7':
			case '8':
			case '9':
				if (_rm->isActive()) {
					break;
				}
				_ui.setTrimmer(_select, key == '0' ? 1.0 : ((key - '0') * 0.1));
				break;
			case osgGA::GUIEventAdapter::KEY_Escape:
				if (!_rm->isActive()) {
					_wm->getViewer()->setDone(true);
				} else {
					if (!_written) {
						_rm->writeOutput(_wm->getOutputPath(), t);
					}
				}
				return true;
			default:
				break;
		}
	} else if (!ea.getHandled() && ea.getEventType() == osgGA::GUIEventAdapter::LEFT_MOUSE_BUTTON) {
		if (_rm->isPointTracing()) {
			_rm->tracePoint(ea.getX(), ea.getY());
			return true;
		}
	}

	if (!ea.getHandled() && ea.getEventType() == osgGA::GUIEventAdapter::CLOSE_WINDOW) {
		osgViewer::View *view = dynamic_cast<osgViewer::View *> (&aa);
		if (view == _wm->getRayView()) {
			// ulozit stav okna
			_wm->saveRayViewState();
		} else if (view == _wm->getSceneView()) {
			// ukoncit aplikaciu
			_wm->getViewer()->setDone(true);
		}
		//		_wm->getViewer()->setDone(true);
	}

	if (!ea.getHandled() && ea.getEventType() == osgGA::GUIEventAdapter::RESIZE) {

		int new_w, new_h;

		new_w = ea.getWindowWidth();
		new_h = ea.getWindowHeight();

		if (new_w != _wm->getSceneWidth() || new_h != _wm->getSceneHeight()) {
			_wm->resize(new_w, new_h);
			//			std::cout << ea.getWindowWidth() << "x" << ea.getWindowHeight() << " ";
		}



	}


	return false;
}


void EventHandler::getUsage(osg::ApplicationUsage& usage) const {
	usage.addKeyboardMouseBinding("r", "start or stop the tracing process");
	usage.addKeyboardMouseBinding("q,e,d", "select trimmer(s)");
	usage.addKeyboardMouseBinding("0-9", "set value for trimmer(s)");
	usage.addKeyboardMouseBinding("a", "cycle through anti-aliasing modes");
	usage.addKeyboardMouseBinding("v", "toggle reflections");
	usage.addKeyboardMouseBinding("b", "toggle refractions (transparency)");
	usage.addKeyboardMouseBinding("m", "export current camera matrix");
	usage.addKeyboardMouseBinding("c", "save the output (if finished)");
	usage.addKeyboardMouseBinding("p", "start per-point tracing (debug mode)");
}

