/* ****************************************************************************
 *
 * rayme - ray tracing application for virtual 3D scenes 
 *
 * ----------------------------------------------------------------------------
 *
 * Andrej Rypak, xrypak@gmail.com, 2011 - 2012
 *
 * Thsesis project - Raytracing of Virtual Graphics Scenes
 * Brno University of Technology, Faculty of Information Technologies
 * Andrej Rypak, xrypak00@stud.fit.vutbr.cz, 2011/2012,
 * under supervision of Jan Peciva, PhD.
 *
 ******************************************************************************/


#include <osg/ValueObject>
#include <sstream>
#include <iostream>
#include "PhotorealismData.h"


using namespace Rayme;

PhotorealismData::PhotorealismData(const osg::Node* node) {
	include(node);
}


PhotorealismData::PhotorealismData() {
}


PhotorealismData::~PhotorealismData() {
}


void PhotorealismData::include(const osg::Node *node) {
	if (node) {
		// taken and modified
		// source: Lexolights - lighting/ShadowVolume.cpp
		std::string str;
		node->getUserValue< std::string > ("Photorealism", str);
		if (!str.empty()) {
			std::stringstream in(str);
			std::string currentKey;
			std::string currentValue;
			while (in) {
				std::string s;
				in >> s;
				if (s.empty()) {
					continue;
				}
				char c = s[0];
				bool isKey = !((c >= '0' && c <= '9') || c == '.' || c == '"' || c == '\'');
				if (isKey) {
					if (!currentKey.empty()) {
						_data.insert(std::make_pair(currentKey, currentValue));
					}
					currentKey = s;
					currentValue = "";
				} else {
					if (currentValue.empty()) {
						currentValue = s;
					} else {
						currentValue += ' ' + s;
					}
				}
			}
			if (!currentKey.empty()) {
				_data.insert(std::make_pair(currentKey, currentValue));
			}
		}
	} else {
		std::cout << "Error: invalid node for photorealism data extractor." << std::endl;
	}

}


bool PhotorealismData::contains(const std::string& key) {
	std::map< std::string, std::string >::iterator it = _data.find(key);
	if (it != _data.end()) {
		return true;
	}
	return false;
}


std::string PhotorealismData::get(const std::string& key) {
	std::map< std::string, std::string >::iterator it = _data.find(key);
	if (it != _data.end()) {
		return it->second;
	}
	return "";
}


std::map< std::string, std::string > PhotorealismData::getData() const {
	return _data;
}

