/* ****************************************************************************
 *
 * rayme - ray tracing application for virtual 3D scenes 
 *
 * ----------------------------------------------------------------------------
 *
 * Andrej Rypak, xrypak@gmail.com, 2011 - 2012
 *
 * Thsesis project - Raytracing of Virtual Graphics Scenes
 * Brno University of Technology, Faculty of Information Technologies
 * Andrej Rypak, xrypak00@stud.fit.vutbr.cz, 2011/2012,
 * under supervision of Jan Peciva, PhD.
 *
 ******************************************************************************/


#include <osg/MatrixTransform>

#include "PrimaryRay.h"


using namespace Rayme;


PrimaryRay::PrimaryRay(osg::Vec3d start, osg::Vec3d end, ushort pixel_x, ushort pixel_y, uchar maxDepth)
: RayIntersector(start, end, 1.0, 0, maxDepth), _x(pixel_x), _y(pixel_y), _samplesPerPixel(1) {
	this->setIntersectionLimit(osgUtil::Intersector::LIMIT_NEAREST);
}


PrimaryRay::PrimaryRay(osg::Vec3d start, osg::Vec3d end, ushort pixel_x, ushort pixel_y, uchar maxDepth, uchar samplesPerPixel)
: RayIntersector(start, end, 1.0, 0, maxDepth), _x(pixel_x), _y(pixel_y), _samplesPerPixel(samplesPerPixel) {
	this->setIntersectionLimit(osgUtil::Intersector::LIMIT_NEAREST);
}


PrimaryRay::~PrimaryRay() {
}


ushort PrimaryRay::getX() {
	return x();
}


ushort PrimaryRay::getY() {
	return y();
}


ushort PrimaryRay::x() {
	return _x;
}


ushort PrimaryRay::y() {
	return _y;
}


uchar PrimaryRay::samplesPerPixel() const {
	return _samplesPerPixel > 0 ? _samplesPerPixel : 1;
}


bool PrimaryRay::isMultiSample() const {
	return _samplesPerPixel > 1;
}


void PrimaryRay::selfTrace(Tracer const& tracer) {
	osgUtil::IntersectionVisitor intersectionVisitor(this);
	intersectionVisitor.setUseKdTreeWhenAvailable(true);
	tracer.getScene()->accept(intersectionVisitor);
	setTraced();
}
