/* ****************************************************************************
 *
 * rayme - ray tracing application for virtual 3D scenes 
 *
 * ----------------------------------------------------------------------------
 *
 * Andrej Rypak, xrypak@gmail.com, 2011 - 2012
 *
 * Thsesis project - Raytracing of Virtual Graphics Scenes
 * Brno University of Technology, Faculty of Information Technologies
 * Andrej Rypak, xrypak00@stud.fit.vutbr.cz, 2011/2012,
 * under supervision of Jan Peciva, PhD.
 *
 ******************************************************************************/


#ifndef RAYME_PRIMARYRAY_H
#define	RAYME_PRIMARYRAY_H

#include <osg/Vec3d>

#include "common.h"
#include "RayIntersector.h"

namespace Rayme {
	class PrimaryRay : public RayIntersector {
	public:
		PrimaryRay(osg::Vec3d start, osg::Vec3d end, ushort pixel_x, ushort pixel_y, uchar maxDepth);
		PrimaryRay(osg::Vec3d start, osg::Vec3d end, ushort pixel_x, ushort pixel_y, uchar maxDepth, uchar samplesPerPixel);
		virtual ~PrimaryRay();

	public:
		void selfTrace(Tracer const& tracer);
		/**
		 * Returns the X coordinate.
		 * These coordinates are used to pair a primary ray with a pixel in an output buffer.
		 */
		ushort getX();
		/**
		 * Returns the Y coordinate.
		 * These coordinates are used to pair a primary ray with a pixel in an output buffer.
		 */
		ushort getY();
		/**
		 * Returns the X coordinate.
		 * These coordinates are used to pair a primary ray with a pixel in an output buffer.
		 */
		ushort x();
		/**
		 * Returns the Y coordinate.
		 * These coordinates are used to pair a primary ray with a pixel in an output buffer.
		 */
		ushort y();
		uchar samplesPerPixel() const;
		bool isMultiSample() const;

	protected:
		ushort _x;
		ushort _y;
		uchar _samplesPerPixel;

	};
}

#endif	/* RAYME_PRIMARYRAY_H */

