/* ****************************************************************************
 *
 * rayme - ray tracing application for virtual 3D scenes 
 *
 * ----------------------------------------------------------------------------
 *
 * Andrej Rypak, xrypak@gmail.com, 2011 - 2012
 *
 * Thsesis project - Raytracing of Virtual Graphics Scenes
 * Brno University of Technology, Faculty of Information Technologies
 * Andrej Rypak, xrypak00@stud.fit.vutbr.cz, 2011/2012,
 * under supervision of Jan Peciva, PhD.
 *
 ******************************************************************************/


#ifndef RAYME_REFRACTEDRAY_H
#define	RAYME_REFRACTEDRAY_H

#include "common.h"
#include "SecondaryRay.h"

namespace Rayme {
	class RefractedRay : public SecondaryRay {
	public:
		RefractedRay(double refractiveIndex, osg::Vec3d start, osg::Vec3d direction, uchar currentDepth, uchar maxDepth);
		RefractedRay(double refractiveIndex, osg::Vec3d start, osg::Vec3d direction, double weight, uchar currentDepth, uchar maxDepth);
		virtual ~RefractedRay();
	protected:
		RefractedRay(double refractiveIndex, osg::Vec3d start, osg::Vec3d direction, double weight, uchar currentDepth, uchar maxDepth, uchar currentInternalDepth);

	public:
		virtual void selfTrace(Tracer const& tracer);
		virtual RefractedRay* evaluate();

	protected:
		/**
		 * This is the refractive index of the environment the refracted ray is running through.
		 */
		double _n1;
		/**
		 * An internal counter for recursive total-internal-reflection reflections.
		 */
		uchar _internalDepth;

	};
}
#endif	/* RAYME_REFRACTEDRAY_H */

