/* ****************************************************************************
 *
 * rayme - ray tracing application for virtual 3D scenes 
 *
 * ----------------------------------------------------------------------------
 *
 * Andrej Rypak, xrypak@gmail.com, 2011 - 2012
 *
 * Thsesis project - Raytracing of Virtual Graphics Scenes
 * Brno University of Technology, Faculty of Information Technologies
 * Andrej Rypak, xrypak00@stud.fit.vutbr.cz, 2011/2012,
 * under supervision of Jan Peciva, PhD.
 *
 ******************************************************************************/


#ifndef RAYME_TRACER_H
#define	RAYME_TRACER_H

#include <osgUtil/LineSegmentIntersector>
#include "common.h"
#include "LightSourceVisitor.h"
#include "RayIntersector.h"

namespace Rayme {
	class RayIntersector;
	class Tracer {
	public:
		Tracer(osg::Node* scene, WorldLightSourceContainer* lc, osgUtil::IntersectorGroup* rays);
		Tracer(osg::Node* scene, WorldLightSourceContainer* lc);
		virtual ~Tracer();

	public:
		osgUtil::IntersectorGroup *getRays() const;
		void setRays(osgUtil::IntersectorGroup* rays);
		void addRay(RayIntersector* ray);
		void reset();
		void setScene(osg::Node* scene);
		osg::Node* getScene() const;
		void setWorldLightSourceContainer(WorldLightSourceContainer* lc);
		WorldLightSourceContainer* getWorldLightSourceContainer() const;
		osg::Timer_t getTotalTime() const;
		void writeImage(osg::Image *img) const;
		virtual void trace();
		virtual void traceSingle(RayIntersector* ray);
		virtual uchar getProgress() const;
		virtual uint getRayCount() const;
		virtual uint getFinishedCount() const;
		virtual osg::Vec3d getEye() const;

	protected:
		void init();
		virtual void evalLoop();
		void setImagePointColor(osg::Image *img, const uint x, const uint y, const osg::Vec4 color) const;
		void setImagePointColorFtoD(osg::Image *img, const uint x, const uint y, const osg::Vec4 color) const;
		void addImagePointColorFtoD(osg::Image *img, const uint x, const uint y, const osg::Vec4 color) const;
		void setEye();

	protected:
		osg::ref_ptr<osg::Node> _scene;
		WorldLightSourceContainer* _lightSourceContainer;
		osg::ref_ptr<osgUtil::IntersectorGroup> _rays;
		osg::Vec3d _eye;
		bool _eye_set;
		osg::Timer_t _totaltime;
		uint _finishedRayCount;
	};
}

#endif	/* RAYME_TRACER_H */

