/* ****************************************************************************
 *
 * rayme - ray tracing application for virtual 3D scenes 
 *
 * ----------------------------------------------------------------------------
 *
 * Andrej Rypak, xrypak@gmail.com, 2011 - 2012
 *
 * Thsesis project - Raytracing of Virtual Graphics Scenes
 * Brno University of Technology, Faculty of Information Technologies
 * Andrej Rypak, xrypak00@stud.fit.vutbr.cz, 2011/2012,
 * under supervision of Jan Peciva, PhD.
 *
 ******************************************************************************/


#include "TracerThread.h"
#include "RayIntersector.h"

// behaviour macros
//#define PRINT_THREAD_INFO // threads will print out information about their current state


using namespace Rayme;

TracerThread::TracerThread(uint threadNumber, osg::Node* scene, WorldLightSourceContainer* lc, osgUtil::IntersectorGroup* rays)
: Tracer(scene, lc, rays), _active(false), _cancelled(false), _threadno(threadNumber) {
}


TracerThread::TracerThread(uint threadNumber, osg::Node* scene, WorldLightSourceContainer* lc)
: Tracer(scene, lc), _active(false), _cancelled(false), _threadno(threadNumber) {
}



TracerThread::~TracerThread() {
	cancel();
}


int TracerThread::cancel() {
	//	if (isRunning()) {
	//		deactivate();
	//	}
	// then wait for the the thread to stop running.
	//	while (isRunning()) {
	//		OpenThreads::Thread::YieldCurrentThread();
	//	}
	_cancelled.exchange(1);
	return 0;
}


void TracerThread::run() {

	if (!isActive()) {

#ifdef PRINT_THREAD_INFO
		std::cout << "thread " << _threadno << " active" << std::endl;
#endif

		activate();

		YieldCurrentThread();

		// do it !
		trace();

#ifdef PRINT_THREAD_INFO
		std::cout << "thread " << _threadno << " finished" << std::endl;
#endif

		deactivate();
	} else {
#ifdef PRINT_THREAD_INFO
		std::cout << "threading ERROR: thread " << _threadno << " still running !" << std::endl;
#endif
	}

}


void TracerThread::evalLoop() {
	// cycle through the intersectors and respective intersections
	osgUtil::IntersectorGroup::Intersectors& intersectors = _rays->getIntersectors();
	for (osgUtil::IntersectorGroup::Intersectors::iterator intersector_itr = intersectors.begin(); intersector_itr != intersectors.end(); ++intersector_itr) {

		if (isCancelled()) {
			break;
		}

		RayIntersector* ray = dynamic_cast<RayIntersector*> ((*intersector_itr).get());

		if (ray) {
			ray->evaluate();
		}

		//		++_finishedRayCount;
		_finishedRayCount.exchange(_finishedRayCount + 1);

	}//foreach intersector
}


void TracerThread::activate() {
	_active.exchange(1);
}


void TracerThread::deactivate() {
	_active.exchange(0);
	_cancelled.exchange(0);
}


bool TracerThread::isActive() const {
	return (bool) _active;
}


bool TracerThread::isCancelled() const {
	return (bool) _cancelled;
}


uchar TracerThread::getProgress() const {
	return _finishedRayCount * 100 / getRayCount();
}


uint TracerThread::getFinishedCount() const {
	return _finishedRayCount;
}


uint TracerThread::getRayCount() const {
	return _rays->getIntersectors().size();
}
