/* ****************************************************************************
 *
 * rayme - ray tracing application for virtual 3D scenes 
 *
 * ----------------------------------------------------------------------------
 *
 * Andrej Rypak, xrypak@gmail.com, 2011 - 2012
 *
 * Thsesis project - Raytracing of Virtual Graphics Scenes
 * Brno University of Technology, Faculty of Information Technologies
 * Andrej Rypak, xrypak00@stud.fit.vutbr.cz, 2011/2012,
 * under supervision of Jan Peciva, PhD.
 *
 ******************************************************************************/


#ifndef RAYME_TRACERTHREAD_H
#define	RAYME_TRACERTHREAD_H

#include <OpenThreads/Thread>
#include <OpenThreads/Atomic>
#include "common.h"
#include "Tracer.h"
#include "LightSourceVisitor.h"

namespace Rayme {

class TracerThread : public Tracer, public OpenThreads::Thread {
public:
	TracerThread(uint threadNumber, osg::Node* scene, WorldLightSourceContainer* lc, osgUtil::IntersectorGroup* rays);
	TracerThread(uint threadNumber, osg::Node* scene, WorldLightSourceContainer* lc);
	virtual ~TracerThread();

public:
	virtual int cancel();
	virtual void run();
	bool isActive() const;
	virtual uchar getProgress() const;
	virtual uint getRayCount() const;
	virtual uint getFinishedCount() const;

protected:
	void activate();
	void deactivate();
	bool isCancelled() const;
	virtual void evalLoop();

protected:
	OpenThreads::Atomic _active;
	OpenThreads::Atomic _cancelled;
	OpenThreads::Atomic _finishedRayCount;
	uint _threadno;

};
}

#endif	/* RAYME_TRACERTHREAD_H */

