/* ****************************************************************************
 *
 * rayme - ray tracing application for virtual 3D scenes 
 *
 * ----------------------------------------------------------------------------
 *
 * Andrej Rypak, xrypak@gmail.com, 2011 - 2012
 *
 * Thsesis project - Raytracing of Virtual Graphics Scenes
 * Brno University of Technology, Faculty of Information Technologies
 * Andrej Rypak, xrypak00@stud.fit.vutbr.cz, 2011/2012,
 * under supervision of Jan Peciva, PhD.
 *
 ******************************************************************************/


#ifndef RAYME_UILAYER_H
#define	RAYME_UILAYER_H

#include <osg/ArgumentParser>
#include "WindowManager.h"
#include "RayManager.h"
#include "common.h"

namespace Rayme {
	class WindowManager;
	struct UIState {
		bool interactive;
		osg::ref_ptr<osg::Node> scene;
		//		osg::Node* scene;
		ushort screenWidth;
		ushort screenHeight;
		std::string outputPath;
		osg::ref_ptr<osg::Camera> initialCamera;
		//		osg::Camera* initialCamera;
		uchar samplesPerPixel;
		ushort threadCount;
		uint chunkSize;
		uint recursionDepth;
		bool phong, textures, bilinearFiltering, recursion, reflections, refractions, selfShadows;
		double reflectionTrimmer, refractionTrimmer;
		double secondaryRayAttenuation_constant, secondaryRayAttenuation_linear, secondaryRayAttenuation_quadratic; //TODO
	};
	class UILayer {
	public:
		UILayer();
		UILayer(RayManager* rm, WindowManager *wm);
		UILayer(int &argc, char** argv, RayManager* rm, WindowManager *wm);
		UILayer(int &argc, char** argv);
		virtual ~UILayer();
	public:
		UILayer* setWindowManager(WindowManager *wm);
		UILayer* setRayManager(RayManager *rm);
		WindowManager* getWindowManager() const;
		RayManager* getRayManager() const;
		//		virtual bool handle(const osgGA::GUIEventAdapter& ea, osgGA::GUIActionAdapter& aa);
		const UIState& getUIState() const;
		void parseArguments(int &argc, char** argv);
		osg::ArgumentParser& getArguments() const;
		bool configureRM();

	public: // ray manager interface wrapper
		uchar getProgress() const;
		bool setRecursionDepth(uchar depth);
		bool setThreadCount(ushort x);
		bool enableTextures(bool x = true);
		bool enableBilinearFiltering(bool x = true);
		bool enablePhongShading(bool x = true);
		bool enableRecursion(bool x = true);
		bool enableReflections(bool x = true);
		bool enableRefractions(bool x = true);
		bool enableSelfShadows(bool x = true);
		bool setMultisampling(uchar samples);
		bool setChunkSize(uint x);
		bool setTrimmer(uchar trimmer, double value);
		char getError() const {
			return _error;
		}

	protected:
		void setDefaults();
		//		void bindWM();

	protected:
		UIState _state;
		osg::ArgumentParser* _arguments;
		RayManager* _rm;
		WindowManager* _wm;
		char _error;
	};
}
#endif	/* RAYME_UILAYER_H */

