/* ****************************************************************************
 *
 * rayme - ray tracing application for virtual 3D scenes 
 *
 * ----------------------------------------------------------------------------
 *
 * Andrej Rypak, xrypak@gmail.com, 2011 - 2012
 *
 * Thsesis project - Raytracing of Virtual Graphics Scenes
 * Brno University of Technology, Faculty of Information Technologies
 * Andrej Rypak, xrypak00@stud.fit.vutbr.cz, 2011/2012,
 * under supervision of Jan Peciva, PhD.
 *
 ******************************************************************************/


#ifndef RAYME_WINDOWMANAGER_H
#define	RAYME_WINDOWMANAGER_H


#include <osg/Texture2D>
#include <osgViewer/Viewer>
#include <osgViewer/CompositeViewer>

#include "common.h"
#include "UILayer.h"

#define IMAGE_FORMAT GL_RGBA
#define IMAGE_TYPE GL_UNSIGNED_BYTE

namespace Rayme {

	class UILayer;
	//
	/**
	 * 
	 */
	class WindowManager {
	public:
		WindowManager(UILayer* ui);
		virtual ~WindowManager();
		enum SelectView {
			VIEW_SCENE = 0x01, VIEW_RAY = 0x02, VIEW_BOTH = 0x03
		};
		struct {
		} Options;

	public:
		void addEventHandler(osgGA::GUIEventHandler* eventHandler, SelectView select);
		void saveRayViewState();
		void prepareBuffers();
		void openRayWindow();
		void closeRayWindow();
		void resize(uint width, uint height);
		int run();

	public: // public getters
		osgViewer::CompositeViewer *getViewer() {
			return _viewer.get();
		}
		osgViewer::View *getSceneView() {
			return _sceneView.get();
		}
		osg::Image *getSceneImage() {
			return _sceneImage.get();
		}
		osg::Texture2D *getSceneTexture() {
			return _sceneTexture.get();
		}
		osgViewer::View *getRayView() {
			return _rayView.get();
		}
		osg::Image *getRayImage() {
			return _rayImage.get();
		}
		osg::Texture2D *getRayTexture() {
			return _rayTexture.get();
		}
		int getSceneWidth() {
			return _sceneWidth;
		}
		int getSceneHeight() {
			return _sceneHeight;
		}
		const std::string getOutputPath() {
			return _outputPath;
		}

	protected:
		void setManipulatorsAndEventHandlers(osgViewer::CompositeViewer *cv, osg::ArgumentParser &arguments);
		osg::Node* createPolygonTextureScreen(osg::Texture2D *outputTexture);
		//	void adjustViewports(uint width, uint height);
		void resizeSceneBuffers(uint width, uint height);
		void resizeRayBuffers(uint width, uint height);


	protected:
		osg::ref_ptr<osgViewer::CompositeViewer> _viewer;
		osg::ref_ptr<osgViewer::View> _sceneView;
		osg::ref_ptr<osg::Image> _sceneImage;
		osg::ref_ptr<osg::Texture2D> _sceneTexture;
		osg::ref_ptr<osgViewer::View> _rayView;
		osg::ref_ptr<osg::Image> _rayImage;
		osg::ref_ptr<osg::Texture2D> _rayTexture;
		osg::GraphicsContext::GraphicsContext::Traits* _raySavedTraits;
		uint _sceneWidth;
		uint _sceneHeight;
		bool _resizeRayWindow;
		std::string _outputPath;

	};
	
	
}

#endif	/* RAYME_ */

